#Read functions are used inside an abstract base class
#and inheretence is used to define readers for three data types

import matplotlib.pyplot as plt
import numpy as np
import glob

#This is the base class, it makes sense to include most of the
#functionality here as both column and binary are very similar
class postproc():

    # Constructor for class, designed for particular folder,
    # header name and file type
    def __init__(self, foldername, headername, basename):

        self.foldername = foldername
        self.headername = headername
        self.basename = basename
        self.files = self.get_files()
        self.header = self.read_header()

    #Read header data
    def read_header(self):
        header = {}
        f = open(self.foldername + self.headername)
        for l in f.readlines():
            key, value = l.split()
            #Reading mixed int/float, try and ask forgiveness...
            try:
                header[key] = int(value)
            except ValueError: 
                header[key] = float(value)
        return header


    #Read files in folder
    def get_files(self):
        files = glob.glob(self.foldername + self.basename + "*")
        files.sort()
        return files

    #Number of records in file
    def get_Nrecs(self):
        return len(self.files)

    #Read file number
    def read_file(self, filename):
        raise NotImplemented

    def read_field(self, recno):
        if recno > self.get_Nrecs():
            raise IndexError("Error request record is greater than"
                             + " max record no. " +  str(self.get_Nrecs()))
        data = self.read_file(self.files[recno])
        field = data.reshape(self.header['Nx'],
                             self.header['Nz'])

        return field.T

#Binary simply needs to redefine read_file
class postproc_binary(postproc):

    #Read file number
    def read_file(self, filename):
        return np.fromfile(open(filename,'rb'), dtype='d')

#Column simply needs to redefine read_file
class postproc_column(postproc):

    #Read file number
    def read_file(self, filename):
        return np.genfromtxt(filename)

#Text needs to redefine read_file and header
class postproc_text(postproc):

    #Read header data
    def read_header(self):
        f = open(self.foldername + self.headername)
        filestr = f.read()
        indx = filestr.find("FoamFile")
        header = {}
        for l in filestr[indx:].split("\n")[2:]:
            if l is "}":
                break
            key, value = l.strip(";").split()
            try:
                header[key] = int(value)
            except ValueError: 
                header[key] = float(value)
        return header

    #Read file number
    def read_file(self, filename):
        f = open(filename)
        filestr = f.read()
        indx = filestr.find("internalField")
        return np.array(filestr[indx:].split("\n")[3:-3], dtype="d")

#Read Text, binary or column data based on input string
#====================
datasource = "binary"
#====================
if datasource is "text":
    pp = postproc_text('./'+datasource+'/', 'filename00000', 'filename')
elif datasource is "column":
    pp = postproc_column('./'+datasource+'/', 'header', 'filename')
elif datasource is "binary":
    pp = postproc_binary('./'+datasource+'/', 'header', 'filename')

#Interface is the same for all objects
print("Datasource is " + datasource)
for i in range(pp.get_Nrecs()):
    f = pp.read_field(i)
    plt.imshow(f)
    plt.colorbar()
    plt.show()



